package com.ruoyi.agent.service;

import com.alibaba.fastjson2.JSONObject;
import dev.langchain4j.data.document.Document;
import dev.langchain4j.data.document.parser.apache.pdfbox.ApachePdfBoxDocumentParser;
import dev.langchain4j.data.document.splitter.DocumentByParagraphSplitter;
import dev.langchain4j.data.segment.TextSegment;
import dev.langchain4j.model.embedding.EmbeddingModel;

import io.milvus.client.MilvusServiceClient;
import io.milvus.grpc.MutationResult;
import io.milvus.grpc.SearchResults;
import io.milvus.param.ConnectParam;
import io.milvus.param.R;
import io.milvus.param.dml.InsertParam;
import io.milvus.param.dml.SearchParam;
import io.milvus.response.SearchResultsWrapper;
import okhttp3.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

public class LangChainMilvusService {
    private String LLM_SERVICE_URL;
    private MilvusServiceClient milvusClient;
    private String collectionName;
    private EmbeddingModel embeddingModel;

    /**
     * 连接milvus知识库指定collection，加入langchain自带emdding
     */
    public LangChainMilvusService(String host, int port, String collectionName, EmbeddingModel embeddingModel) {
        this.milvusClient = new MilvusServiceClient(
                ConnectParam.newBuilder()
                        .withHost(host)
                        .withPort(port)
                        .build()
        );
        this.LLM_SERVICE_URL = "http://" + host + ":8000/generate";
        this.collectionName = collectionName;
        this.embeddingModel = embeddingModel;
    }

    public void insertLangchainEmbeddingDocument(File file) throws IOException {
        // 加载文档
        InputStream fileInputStream = new FileInputStream(file);
        Document document = new ApachePdfBoxDocumentParser().parse(fileInputStream);
        DocumentByParagraphSplitter splitter = new DocumentByParagraphSplitter(150,10);;
        List<TextSegment> segments = splitter.split(document);

        // 提取文本和生成嵌入
        List<String> texts = new ArrayList<>();
        List<List<Float>> embeddings = new ArrayList<>();

        for (TextSegment segment : segments) {
            String text = segment.text();
            if (text.trim().isEmpty())
                continue;
            texts.add(text);
            embeddings.add(embeddingModel.embed(text).content().vectorAsList());
        }

        // 准备插入数据
        List<InsertParam.Field> fields = new ArrayList<>();
        fields.add(new InsertParam.Field("file_name", Arrays.asList(file.getName())));
        fields.add(new InsertParam.Field("content", Arrays.asList(texts)));
        fields.add(new InsertParam.Field("embedding", Arrays.asList(embeddings)));

        InsertParam insertParam = InsertParam.newBuilder()
                .withCollectionName(collectionName)
                .withFields(fields)
                .build();

        // 执行插入
        R<MutationResult> insertResult = milvusClient.insert(insertParam);

        if (insertResult.getStatus() != R.Status.Success.getCode()) {
            throw new RuntimeException("Failed to insert document: " + insertResult.getMessage());
        }
    }

    // 从Milvus检索相关文档
    public List<String> retrieveFromMilvus(String query, int topK) throws IOException {
        List<Float> queryVector = embeddingModel.embed(query).content().vectorAsList();

        SearchParam searchParam = SearchParam.newBuilder()
                .withCollectionName(collectionName)
                .withVectors(queryVector)
                .withTopK(topK)
                .withOutFields(Arrays.asList("content"))
                .build();

        R<SearchResults> response = milvusClient.search(searchParam);
        SearchResultsWrapper wrapper = new SearchResultsWrapper(response.getData().getResults());

        return wrapper.getRowRecords(0).stream()
                .map(record -> (String) record.get("content"))
                .collect(Collectors.toList());
    }

    // 调用LLM+RAG生成回答
    public String generateAnswerWithRag(String question, List<String> contexts) throws IOException {
        String prompt = buildPrompt(question, contexts);
        // 构建带动态参数的URL
        HttpUrl url = HttpUrl.parse(LLM_SERVICE_URL)
                .newBuilder()
                .addQueryParameter("prompt", prompt)
                .addQueryParameter("max_token", String.valueOf(512))
                .build();

        Request request = new Request.Builder()
                .url(url)
                .build();

        OkHttpClient client = new OkHttpClient.Builder()
                .readTimeout(60, TimeUnit.SECONDS)      // 读取响应超时
                .build();

        try (Response response = client.newCall(request).execute()) {
            String responseBody = response.body().string();
            String responseResult = JSONObject.parseObject(responseBody).getString("generated_text");
            return responseResult;
        }
    }

    private String buildPrompt(String question, List<String> contexts) {
        StringBuilder sb = new StringBuilder();
        sb.append("根据以下上下文回答问题：\n\n");
        for (int i = 0; i < contexts.size(); i++) {
            sb.append("上下文").append(i+1).append(": ").append(contexts.get(i)).append("\n\n");
        }
        sb.append("问题: ").append(question).append("\n回答: ");
        return sb.toString();
    }

    public void close() {
        milvusClient.close();
    }
}