package com.ruoyi.agent.service;

import com.alibaba.fastjson2.JSONObject;
import dev.langchain4j.data.document.Document;
import dev.langchain4j.data.document.parser.apache.pdfbox.ApachePdfBoxDocumentParser;
import dev.langchain4j.data.document.splitter.DocumentByParagraphSplitter;
import dev.langchain4j.data.segment.TextSegment;
import dev.langchain4j.model.embedding.EmbeddingModel;
import io.milvus.client.MilvusServiceClient;
import io.milvus.grpc.DataType;
import io.milvus.grpc.MutationResult;
import io.milvus.grpc.SearchResults;
import io.milvus.param.ConnectParam;
import io.milvus.param.IndexType;
import io.milvus.param.MetricType;
import io.milvus.param.R;
import io.milvus.param.collection.CreateCollectionParam;
import io.milvus.param.collection.DropCollectionParam;
import io.milvus.param.collection.FieldType;
import io.milvus.param.dml.InsertParam;
import io.milvus.param.dml.SearchParam;
import io.milvus.param.highlevel.collection.ListCollectionsParam;
import io.milvus.param.index.CreateIndexParam;
import io.milvus.response.SearchResultsWrapper;
import okhttp3.HttpUrl;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

public class MilvusService {
    private String LLM_SERVICE_URL;
    private MilvusServiceClient milvusClient;
    private String collectionName;
    private EmbeddingModel embeddingModel;

    /**
     *  连接milvus知识库
     */
    public MilvusService(String host, int port) {
        this.milvusClient = new MilvusServiceClient(
                ConnectParam.newBuilder()
                        .withHost(host)
                        .withPort(port)
                        .build()
        );
    }

    /**
     * 连接milvus知识库指定collection
     */
    public MilvusService(String host, int port, String collectionName) {
        this.milvusClient = new MilvusServiceClient(
                ConnectParam.newBuilder()
                        .withHost(host)
                        .withPort(port)
                        .build()
        );
        this.collectionName = collectionName;
    }

    /**
     * 连接milvus知识库指定collection，加入langchain自带emdding
     */
    public MilvusService(String host, int port, String collectionName, EmbeddingModel embeddingModel) {
        this.milvusClient = new MilvusServiceClient(
                ConnectParam.newBuilder()
                        .withHost(host)
                        .withPort(port)
                        .build()
        );
        this.collectionName = collectionName;
        this.embeddingModel = embeddingModel;
    }

    /**
     * 新建知识库Collection（含Schema、Field、Index）
     */
    public void createCollection(int dimension) {
        FieldType idField = FieldType.newBuilder()
                .withName("id")
                .withDataType(DataType.Int64)
                .withPrimaryKey(true)
                .withAutoID(true)
                .build();

        FieldType fileNameField = FieldType.newBuilder()
                .withName("file_name")
                .withDataType(DataType.VarChar)
                .withMaxLength(256)
                .build();

        FieldType contentField = FieldType.newBuilder()
                .withName("content")
                .withDataType(DataType.VarChar)
                .withMaxLength(65535)
                .build();

        FieldType vectorField = FieldType.newBuilder()
                .withName("embedding")
                .withDataType(DataType.FloatVector)
                .withDimension(dimension)
                .build();

        CreateCollectionParam createCollectionParam = CreateCollectionParam.newBuilder()
                .withCollectionName(collectionName)
                .addFieldType(idField)
                .addFieldType(fileNameField)
                .addFieldType(contentField)
                .addFieldType(vectorField)
                .build();

        milvusClient.createCollection(createCollectionParam);

        // 创建索引
        CreateIndexParam createIndexParam = CreateIndexParam.newBuilder()
                .withCollectionName(collectionName)
                .withFieldName("embedding")
                .withIndexType(IndexType.IVF_FLAT)
                .withMetricType(MetricType.COSINE)
                .withExtraParam("{\"nlist\": 64}")
                .build();

        milvusClient.createIndex(createIndexParam);
    }

    /**
     * 查询知识库Collection
     */
    public List<String> getCollectionNames() {
        ListCollectionsParam param = ListCollectionsParam.newBuilder().build();
        return milvusClient.listCollections(param).getData().collectionNames;
    }

    /**
     * 删除知识库Collection
     */
    public void deleteCollectionName(String collectionName) {
        DropCollectionParam param = DropCollectionParam.newBuilder().withCollectionName(collectionName).build();
        milvusClient.dropCollection(param);
    }

    public void close() {
        milvusClient.close();
    }
}